import traceback
import time
import threading
import multiprocessing as mp
from queue import Empty

from pyjoystick.stash import Stash
from pyjoystick.utils import PeriodicThread
from pyjoystick.run_thread import ThreadEventManager


class MultiprocessingEventManager(ThreadEventManager):
    def __init__(self, event_loop=None, add_joystick=None, remove_joystick=None, handle_key_event=None, alive=None,
                 button_repeater=None, activity_timeout=0.01):
        if alive is None:
            alive = mp.Event()
        self.proc = None
        self.queue = mp.Queue()
        super().__init__(event_loop=event_loop, add_joystick=add_joystick, remove_joystick=remove_joystick,
                         handle_key_event=handle_key_event, alive=alive, button_repeater=button_repeater,
                         activity_timeout=activity_timeout)

    def send_cmd(self, name, *args, **kwars):
        """Send a command to the main process."""
        self.queue.put((name, args, kwars))

    def _save_joystick(self, joy):
        self.save_joystick(joy)
        self.send_cmd('save_joystick', joy)

    def _delete_joystick(self, joy):
        self.delete_joystick(joy)
        self.send_cmd('delete_joystick', joy)

    def _handle_key_event(self, key):
        """Function to handle key event happens"""
        self.send_cmd('handle_key_event', key)

    def process_queue(self):
        """Continually process the Queue data."""
        while self.is_running():
            try:
                func_name, args, kwargs = self.queue.get(timeout=2)
                func = getattr(self, func_name, None)
                if func:
                    func(*args, **kwargs)
            except Empty:
                pass
            except Exception:
                traceback.print_exc()

    def run(self, event_loop, add_joystick, remove_joystick, handle_key_event, alive=None, button_repeater=None,
            queue=None):
        """Run the an event loop to process SDL Events.

        Args:
            event_loop (callable/function): Event loop function to run.
            add_joystick (callable/function): Called when a new Joystick is found!
            remove_joystick (callable/function): Called when a Joystick is removed!
            handle_key_event (callable/function): Called when a new key event occurs!
            alive (callable/function)[None]: Function to return True to continue running. If None run forever
            button_repeater (ButtonRepeater): Thread to start which will monitor button keys and trigger repeating.
            queue (mp.Queue): Queue to communicate with the main process.
        """
        self.alive = alive
        self.button_repeater = button_repeater
        self.queue = queue

        # Run process events on a timer
        self.worker = PeriodicThread(self.activity_timeout, self.process_events)
        self.worker.alive = self.alive  # stop when this event stops
        self.worker.daemon = True
        self.worker.start()

        if button_repeater is not None:
            button_repeater.start()
        event_loop(add_joystick, remove_joystick, handle_key_event, alive=self.is_running)

    def start(self):
        """Start running the event loop."""
        self.stop()

        self.alive.set()
        self.proc = mp.Process(target=self.run,
                               args=(self.event_loop, self._save_joystick, self._delete_joystick, self._handle_key_event),
                               kwargs={'alive': self.alive, 'button_repeater': self.button_repeater,
                                       'queue': self.queue})
        self.proc.daemon = True
        self.proc.start()

        self.worker = threading.Thread(target=self.process_queue)
        self.worker.daemon = True
        self.worker.start()
        return self



import contextlib
import time
import threading

from pyjoystick.stash import Stash
from pyjoystick.utils import PeriodicThread, deadband


class ThreadEventManager(object):

    JOYSTICK_PROXY = None

    def __init__(self, event_loop=None, add_joystick=None, remove_joystick=None, handle_key_event=None, alive=None,
                 button_repeater=None, activity_timeout=1/30):
        super().__init__()

        if alive is None:
            alive = threading.Event()

        self.activity_timeout = activity_timeout
        self._button_repeater = None

        self.event_loop = event_loop
        self.joysticks = Stash()
        self.alive = alive
        self.proc = None
        self.worker = None
        self.event_lock = threading.RLock()
        self.event_buttons = Stash()
        self.event_latest = {}

        if add_joystick is not None:
            self.add_joystick = add_joystick
        if remove_joystick is not None:
            self.remove_joystick = remove_joystick
        if handle_key_event is not None:
            self.handle_key_event = handle_key_event
        if button_repeater is not None:
            self.set_button_repeater(button_repeater)

    def add_joystick(self, joy):
        """Save the added joystick."""
        pass

    def remove_joystick(self, joy):
        """Remove the given joystick."""
        pass

    def handle_key_event(self, key):
        """Function to handle key event happens"""
        pass

    def get_button_repeater(self):
        """Return the button repeater."""
        return self._button_repeater

    def set_button_repeater(self, value):
        """Set the button repeater."""
        self._button_repeater = value
        try:
            self._button_repeater.key_repeated = self._update_key_event
        except:
            pass

    button_repeater = property(get_button_repeater, set_button_repeater)

    def save_joystick(self, joy):
        """Save the added joystick."""
        if self.JOYSTICK_PROXY:
            joy = self.JOYSTICK_PROXY(joy)
        self.joysticks.append(joy)

        # Run the callback handler
        self.add_joystick(joy)

    def delete_joystick(self, joy):
        """Delete the removed joystick."""
        try:
            self.joysticks.remove(joy)
        except:
            pass

        # Run the callback handler
        self.remove_joystick(joy)

    def save_key_event(self, key):
        """Save the initial key event."""
        try:
            joy = self.joysticks[key.joystick]
            key.joystick = joy
        except:
            pass

        # Calculate deadband after a static joystick was set
        if key.keytype == key.AXIS:
            dead = getattr(key.joystick, 'deadband', 0)
            if dead:
                key.value = deadband(key.value, dead)

            # Check if axis is still at 0 (No change due to deadband) to reduce number of events
            try:
                if key.value == 0 and key.joystick.get_axis(key.number) == 0:
                    return
            except (AttributeError, IndexError, Exception):
                pass

        try:
            self.button_repeater.set(key)
        except:
            pass

        self._update_key_event(key)

    def _update_key_event(self, key):
        """Update the event list from the key event."""
        value = key.value
        with self.event_lock:
            try:
                key.joystick.update_key(key)
            except:
                pass

            if key.keytype == key.BUTTON:
                self.event_buttons.append(key)
            else:
                self.event_latest[key] = value

    def clear_key_events(self):
        """Clear all of the key events."""
        with self.event_lock:
            buttons, self.event_buttons = self.event_buttons, Stash()
            latest, self.event_latest = self.event_latest, {}
        return buttons, latest

    def process_events(self):
        """Process all of the saved events."""
        buttons, latest = self.clear_key_events()
        for key in buttons:
            self.handle_key_event(key)
        for key, value in latest.items():
            key.value = value
            self.handle_key_event(key)

    @contextlib.contextmanager
    def run_during(self):
        """Context manager to temporarily run the manager if the manager is not already running."""
        if not self.is_running():
            self.start()
            yield
            self.stop()
        else:
            yield

    def wait(self, conditional=None, timeout=float('inf'), sleep_func=None):
        """Wait for the given timeout or conditional function to return false.

        Args:
            conditional (callable)[None]: Callable that returns True to keep waiting. If raises error stop waiting.
            timeout (float/int)[float('inf')]: Time for when to stop waiting.
            sleep_func (callable)[None]: How to sleep and wait (ex. time.sleep(0.001)). If raises error stop waiting.
        """
        if sleep_func is None:
            sleep_func = lambda: time.sleep(0.01)
        if conditional is None:
            conditional = lambda: True
        elif not callable(conditional):
            conditional = lambda: conditional

        start = time.time()
        try:
            while (time.time() - start) < timeout and conditional():
                sleep_func()
        except (ValueError, TypeError, Exception):  # If any error occurs stop waiting.
            pass

    def find_key(self, joysticks=None, key_types=None, timeout=float("inf"), sleep_func=None):
        """Wait and return the next key that is pressed.

        Args:
            joysticks (list/Joystick)[None]: Joystick(s) to allow events for.
            key_types (list/KeyTypes)[None]: List of key type. Found with Key.KeyTypes.Axis
            timeout (float/int)[float('inf')]: Timeout to wait.
            sleep_func (callable)[None]: How to sleep and wait (time.sleep(0.01)).

        Returns:
            key (Key)[None]: If found return the first Key else return None.
        """
        if joysticks is None:
            joysticks = []
        elif not isinstance(joysticks, list):
            try:
                joysticks = list(joysticks)  # convert tuple to list
            except (TypeError, Exception):
                joysticks = [joysticks]  # Put a single object in a list
        if key_types is None:
            key_types = []

        data = {'found key': None}

        def filter_find_key(key):
            is_joystick = len(joysticks) == 0 or key.joystick in joysticks
            is_key_type = len(key_types) == 0 or key.has_keytype(key.keytype, key_types)
            is_valid_value = abs(key.value) > 0.5
            if is_joystick and is_key_type and is_valid_value and data['found key'] is None:
                data['found key'] = key

        def is_not_found():
            return data['found key'] is None

        # Change the event handler
        old_handle_key_event, self.handle_key_event = self.handle_key_event, filter_find_key

        with self.run_during():
            self.wait(is_not_found, timeout, sleep_func)

        # Reset event handler
        self.handle_key_event = old_handle_key_event

        return data['found key']

    def run(self, event_loop, add_joystick, remove_joystick, handle_key_event, alive=None, button_repeater=None):
        """Run the an event loop to process SDL Events.

        Args:
            event_loop (callable/function): Event loop function to run.
            add_joystick (callable/function): Called when a new Joystick is found!
            remove_joystick (callable/function): Called when a Joystick is removed!
            handle_key_event (callable/function): Called when a new key event occurs!
            alive (callable/function)[None]: Function to return True to continue running. If None run forever
            button_repeater (ButtonRepeater): Thread to start which will monitor button keys and trigger repeating.
        """
        if alive is None:
            alive = lambda: True

        if button_repeater is not None:
            button_repeater.start()

        event_loop(add_joystick, remove_joystick, handle_key_event, alive=alive)

    def is_running(self):
        """Return if the event loop is running."""
        return self.alive.is_set()

    def start(self):
        """Start running the event loop."""
        self.stop()

        self.alive.set()
        self.proc = threading.Thread(target=self.run,
                                     args=(self.event_loop, self.save_joystick, self.delete_joystick,
                                           self.save_key_event),
                                     kwargs={'alive': self.is_running, 'button_repeater': self.button_repeater})
        self.proc.daemon = True
        self.proc.start()

        self.worker = PeriodicThread(self.activity_timeout, self.process_events)
        self.worker.alive = self.alive  # stop when this event stops
        self.worker.daemon = True
        self.worker.start()
        return self

    def stop(self):
        """Stop running the event loop."""
        try:
            self.alive.clear()
        except:
            pass
        try:
            # Try pumping an event on queue to break out of the wait condition
            self.event_loop.stop_event_wait()
        except:
            pass
        try:
            self.proc.join(0)
        except:
            pass
        self.proc = None
        try:
            self.worker.join(0)
        except:
            pass
        self.worker = None
        try:
            self.clear_key_events()
        except:
            pass
        return self

    def __enter__(self):
        if not self.is_running():
            self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()
        return exc_type is None

    def __getstate__(self):
        return {'activity_timeout': self.activity_timeout,
                'button_repeater': self.button_repeater,
                'event_loop': self.event_loop,
                'joysticks': Stash(),
                'alive': self.alive,
                'proc': None,
                'worker': None,
                'event_buttons': Stash(),
                'event_latest': {},
                }

    def __setstate__(self, state):
        for k, v in state.items():
            setattr(self, k, v)

        if getattr(self, 'event_lock', None) is None:
            self.event_lock = threading.RLock()


if __name__ == '__main__':
    import time
    import argparse
    from pyjoystick import Key, ButtonRepeater, HatRepeater, ButtonHatRepeater
    try:
        from pyjoystick.sdl2 import Joystick as SDLJoystick, run_event_loop as run_sdl_loop
    except:
        SDLJoystick = None
        run_sdl_loop = None
    try:
        from pyjoystick.pygame import Joystick as PygameJoystick, run_event_loop as run_pygame_loop
    except:
        PygameJoystick = None
        run_pygame_loop = None

    # Parse the command line arguments
    P = argparse.ArgumentParser(description='Run a thread event loop.')
    P.add_argument('--lib', type=str, default='sdl', choices=['sdl', 'pygame'],
                   help='Library to run the thread event loop with.')
    P.add_argument('--timeout', type=float, default=float('inf'), help='Time to run for')
    P.add_argument('--device', type=int, default=None, help='Device index to monitor.')
    P.add_argument('--keytype', type=str, default=Key.ALL_KEYTYPES,
                   choices=[Key.AXIS, Key.BUTTON, Key.HAT, Key.BALL, Key.ALL_KEYTYPES], help='Key type to monitor.')

    ARGS = P.parse_args()

    TIMEOUT = ARGS.timeout
    DEVICE = ARGS.device
    KEYTYPES = ARGS.keytype

    if ARGS.lib == 'pygame':
        run_event_loop = run_pygame_loop
        Joystick = PygameJoystick
    else:
        run_event_loop = run_sdl_loop
        Joystick = SDLJoystick

    print("Devices:")
    for joy in Joystick.get_joysticks():
        print('\t', '{}.'.format(joy.get_id()), joy.get_name())

    def print_add(joy):
        print('Added', joy, '\n', end='\n', flush=True)

    def print_remove(joy):
        print('Removed', joy, '\n', end='\n', flush=True)

    def key_received(key):
        if DEVICE is None:
            print(key, '==', key.value)
        elif key.joystick == DEVICE:
            DEVICE.update_key(key)

            keys = '\t'.join((format_key(k) for k in DEVICE.keys if k.keytype in KEYTYPES))
            print('\r', keys, end='', flush=True)

    def format_key(key):
        try:
            value = key.value
            if isinstance(value, float):
                value = '{:2.2f}'.format(key.value)
            else:
                value = str(value)
        except:
            if key.value is None:
                value = '0'
            else:
                value = '{:5}'.format(key.value)
        return '{} {} == {:>5}'.format(key.keytype, key.number, value)

    # Run the event loop
    with ThreadEventManager(run_event_loop, print_add, print_remove, key_received, button_repeater=ButtonHatRepeater()):
        try:
            time.sleep(TIMEOUT)
        except:
            while True:
                time.sleep(1000)



from qtpy import QtCore


__all__ = ["MousePanTilt"]


class MousePanTilt(QtCore.QObject):
    """Event Filter for handling mouse gestures and clicks to triggering actions.
    """

    pantilt = QtCore.Signal(float, float)
    zoom = QtCore.Signal(float)

    def __init__(self):
        super().__init__()

        self._mouse_pos = None
    # end Constructor

    def eventFilter(self, receiver, event):
        """Event filter for key press and release setting the key values and sending the signal."""
        etype = event.type()
        try:
            pos = event.posF()
        except AttributeError:
            try:
                pos = event.pos()
            except AttributeError:
                pos = None

        # Check for pantilt
        if etype == QtCore.QEvent.MouseButtonPress:
            self._mouse_pos = pos
            return True

        elif etype == QtCore.QEvent.MouseButtonRelease and self._mouse_pos is not None:
            end_pos = pos
            if self._mouse_pos == end_pos:
                # Calculate offset from center
                center_w = receiver.width() / 2
                center_h = receiver.height() / 2
                pantilt = QtCore.QPointF((self._mouse_pos.x() - center_w) / center_w,
                                         (center_h - self._mouse_pos.y()) / center_h)
            else:
                # Calculate offset dragged
                pantilt = QtCore.QPointF((end_pos.x() - self._mouse_pos.x()) / receiver.width(),
                                         (self._mouse_pos.y() - end_pos.y()) / receiver.height())

            self.pantilt.emit(pantilt.x(), pantilt.y())
            self._mouse_pos = None
            return True

        return super().eventFilter(receiver, event)
    # end eventFilter
# end class MousePanTilt


if __name__ == "__main__":
    from qtpy import QtWidgets, QtGui, QtCore

    app = QtWidgets.QApplication([])

    widg = QtWidgets.QWidget()
    widg.show()
    mouse = MousePanTilt()
    mouse.pantilt.connect(print)
    widg.installEventFilter(mouse)

    app.exec_()



from .stash import Stash


__all__ = ['KeyTypes', 'HatValues', 'Key', 'Joystick']


class KeyTypes:
    AXIS = "Axis"
    BUTTON = "Button"
    HAT = "Hat"
    BALL = "Ball"
    ALL_KEYTYPES = ','.join((AXIS, BUTTON, HAT, BALL))

    @classmethod
    def has_keytype(cls, keytype, key_types):
        try:
            if keytype == cls.ALL_KEYTYPES or str(keytype) in key_types:
                return True
        except (TypeError, ValueError, Exception):
            pass
        return False


class HatValues:
    HAT_CENTERED = 0
    HAT_UP = 1
    HAT_RIGHT = 2
    HAT_DOWN = 4
    HAT_LEFT = 8
    HAT_RIGHTUP = 3
    HAT_RIGHTDOWN = 6
    HAT_LEFTUP = 9
    HAT_LEFTDOWN = 12

    ALL_HAT_VALUES = (HAT_CENTERED | HAT_UP | HAT_RIGHT | HAT_DOWN | HAT_LEFT |
                      HAT_RIGHTUP | HAT_RIGHTDOWN | HAT_LEFTUP | HAT_LEFTDOWN)


class Key(object):
    # Key Types
    KeyTypes = KeyTypes
    AXIS = KeyTypes.AXIS
    BUTTON = KeyTypes.BUTTON
    HAT = KeyTypes.HAT
    BALL = KeyTypes.BALL
    ALL_KEYTYPES = KeyTypes.ALL_KEYTYPES
    has_keytype = staticmethod(KeyTypes.has_keytype)

    # HAT Values (Guessing they are more bit flags than enums.)
    HatValues = HatValues
    HAT_CENTERED = HatValues.HAT_CENTERED
    HAT_UP = HatValues.HAT_UP
    HAT_RIGHT = HatValues.HAT_RIGHT
    HAT_DOWN = HatValues.HAT_DOWN
    HAT_LEFT = HatValues.HAT_LEFT
    HAT_RIGHTUP = HatValues.HAT_RIGHTUP
    HAT_RIGHTDOWN = HatValues.HAT_RIGHTDOWN
    HAT_LEFTUP = HatValues.HAT_LEFTUP
    HAT_LEFTDOWN = HatValues.HAT_LEFTDOWN
    ALL_HAT_VALUES = HatValues.ALL_HAT_VALUES

    def __init__(self, keytype, number, value=None, joystick=None, is_repeat=False, override=False):
        self.keytype = keytype
        self.number = number
        self.value = value
        self.joystick = joystick
        self.is_repeat = is_repeat
        self.override = override

    def get_value(self):
        if self.value is None:
            return 0
        return self.value

    def set_value(self, value):
        self.value = value

    def update_value(self, joystick=None):
        if joystick is None:
            joystick = self.joystick
        try:
            v = joystick.get_key(self).get_value()
            self.value = v
        except:
            pass

    def copy(self):
        return self.__class__(self.keytype, self.number, self.value, self.joystick,
                              is_repeat=False, override=self.override)

    @property
    def keyname(self):
        """Return the keytype and number of the key as a string."""
        return '{} {}'.format(self.keytype, self.number)

    def __str__(self):
        if self.joystick:
            return '{}: {}'.format(self.joystick, self.keyname)
        else:
            return self.keyname

    def __repr__(self):
        if self.joystick:
            return '<{module}.{name} {joystick}: {keyname} at {id}>'.format(
                    module=self.__module__, name=self.__class__.__name__, id=id(self),
                    joystick=self.joystick, keyname=self.keyname)
        else:
            return '<{module}.{name} {keyname} at {id}>'.format(
                    module=self.__module__, name=self.__class__.__name__, id=id(self),
                    joystick=self.joystick, keyname=self.keyname)

    def __hash__(self):
        return hash(str(self))

    def __eq__(self, other):
        try:
            if other.keytype == self.keytype and other.number == self.number:
                # Check if joysticks match if they are not None
                if other.joystick is not None and self.joystick is not None:
                    return other.joystick == self.joystick
                return True
            return False
        except:
            pass
        try:
            return str(self) == str(other)
        except:
            return False


class Joystick(object):
    @classmethod
    def get_joysticks(cls):
        """Return a list of available joysticks."""
        # return []
        raise NotImplementedError

    def __init__(self, *args, **kwargs):
        super().__init__()

        # Optional predefined variables (use with __new__)
        self.joystick = getattr(self, 'joystick', None)  # Internal joystick object

        self.identifier = getattr(self, 'identifier', -1)
        self.name = getattr(self, 'name', '')
        self.numaxes = getattr(self, 'numaxes', -1)
        self.numbuttons = getattr(self, 'numbuttons', -1)
        self.numhats = getattr(self, 'numhats', -1)
        self.numballs = getattr(self, 'numballs', -1)

        self.axis = getattr(self, 'axis', Stash())
        self.button = getattr(self, 'button', Stash())
        self.hat = getattr(self, 'hat', Stash())
        self.ball = getattr(self, 'ball', Stash())
        self.keys = getattr(self, 'keys', Stash(self.axis + self.button + self.hat + self.ball))

        self.deadband = getattr(self, 'deadband', 0.2)

        self.init_keys()

    def init_keys(self):
        """Initialize the keys."""
        self.axis = Stash(Key(Key.AXIS, i, None, self) for i in range(self.get_numaxes()))
        self.button = Stash(Key(Key.BUTTON, i, None, self) for i in range(self.get_numbuttons()))
        self.hat = Stash(Key(Key.HAT, i, None, self) for i in range(self.get_numhats()))
        self.ball = Stash(Key(Key.BALL, i, None, self) for i in range(self.get_numballs()))
        self.keys = Stash(self.axis + self.button + self.hat + self.ball)

    def is_active(self):
        """Return if this joystick is still active and available."""
        raise NotImplementedError

    def close(self):
        """Close the joystick."""
        raise NotImplementedError

    def get_key(self, key):
        """Return the key for the given key."""
        key_attr = getattr(self, str(key.keytype).lower())  # self.axis, self.button, self.hat, or self.ball
        return key_attr[key.number]

    def update_key(self, key):
        """Update the value for a given key."""
        self.get_key(key).set_value(key.value)

    def get_id(self):
        """Return the joystick id."""
        return self.identifier

    def get_name(self):
        """Return the name of the joystick."""
        return self.name

    def get_numaxes(self):
        """Return the number of axes."""
        return self.numaxes

    def get_axis(self, number):
        """Return the current value for the given axes."""
        return self.axis[number].get_value()

    def get_numbuttons(self):
        """Return the number of buttons."""
        return self.numbuttons

    def get_button(self, number):
        """Return the value for the given button number."""
        return self.button[number].get_value()

    def get_numhats(self):
        """Return the number of hats."""
        return self.numhats

    def get_hat(self, number):
        """Return the (hat [0], hat [1]) value for the given hat number."""
        return self.hat[number].get_value()

    def get_numballs(self):
        """Return the number of track balls."""
        return self.numballs

    def get_ball(self, number):
        """Return the current value for the given axes."""
        return self.ball[number].get_value()

    def get_deadband(self):
        """Return the deadband for this joystick axis."""
        return self.deadband

    def set_deadband(self, value):
        """Return the deadband for this joystick axis."""
        self.deadband = value

    def __eq__(self, other):
        name, ident, joystick = self.get_name(), self.get_id(), self.joystick
        try:
            return name == other.get_name() or ident == other or (joystick == other.joystick and joystick is not None)
        except:
            pass
        try:
            return name == other or ident == other or (joystick == other and joystick is not None)
        except:
            pass
        return False

    def __int__(self):
        return self.get_id()

    def __str__(self):
        return self.get_name()

    def __hash__(self):
        return hash('{} {}'.format(self.identifier, self.name))

    def __getstate__(self):
        return {
            'joystick': None,
            'identifier': self.identifier,
            'name': self.name,
            'numaxes': self.numaxes,
            'numbuttons': self.numbuttons,
            'numhats': self.numhats,
            'numballs': self.numballs,

            'axis': self.axis,
            'button': self.button,
            'hat': self.hat,
            'ball': self.ball,
            'keys': self.keys,

            'deadband': self.deadband
            }

    def __setstate__(self, state):
        for k, v in state.items():
            setattr(self, k, v)



from .widget_updater import get_updater, set_updater, cleanup_app, ThreadUpdater
from .widgets import AxisWidget, ButtonWidget, HatWidget, SpinSlider, LED

from .keys_trigger import RESOURCE_DIR, KEYBOARD_ACTIVE_IMG, KEYBOARD_INACTIVE_IMG, \
    BindKeys, WASDController, ArrowController, GimbalController

from .mouse_trigger import MousePanTilt

from .keymapper import JOYSTICK_ACTIVE_IMG, JOYSTICK_INACTIVE_IMG, JoystickKeyMapper



from .__meta__ import version as __version__

from .utils import PYJOYSTICK_DIR, deadband, change_path, rescale, PeriodicThread
from .stash import Stash
from .button_repeater import Repeater, ButtonRepeater, HatRepeater, ButtonHatRepeater
from .interface import Key, Joystick

try:
    from .sdl2 import Joystick as SDLJoystick, run_event_loop as run_sdl_loop
except:
    SDLJoystick = None
    run_sdl_loop = None

from .run_thread import ThreadEventManager
from .run_process import MultiprocessingEventManager



"""
setup.py - Setup file to distribute the library

See Also:
    https://github.com/pypa/sampleproject
    https://packaging.python.org/en/latest/distributing.html
    https://pythonhosted.org/an_example_pypi_project/setuptools.html
"""
import os
import glob
import sys
from setuptools import setup, Extension, find_packages


def read(fname):
    """Read in a file"""
    with open(os.path.join(os.path.dirname(__file__), fname), 'r') as file:
        return file.read()


def get_meta(filename):
    """Return the metadata dictionary from the given filename."""
    with open(filename, 'r') as f:
        meta = {}
        exec(compile(f.read(), filename, 'exec'), meta)
        return meta


if __name__ == "__main__":
    # Variables
    meta = get_meta('pyjoystick/__meta__.py')
    name = meta['name']
    version = meta['version']
    description = meta['description']
    url = meta['url']
    author = meta['author']
    author_email = meta['author_email']
    keywords = 'builder installer project pyinstaller'
    packages = find_packages(exclude=('tests', 'docs', 'bin'))

    # Extensions
    extensions = []
    # module1 = Extension('libname',
    #                     # define_macros=[('MAJOR_VERSION', '1')],
    #                     # extra_compile_args=['-std=c99'],
    #                     sources=['file.c', 'dir/file.c'],
    #                     include_dirs=['./dir'])
    # extensions.append(module1)

    setup(name=name,
          version=version,
          description=description,
          long_description=read('README.rst'),
          keywords=keywords,
          url=url,
          download_url=''.join((url, '/archive/v', version, '.tar.gz')),

          author=author,
          author_email=author_email,

          license='MIT',
          platforms='any',
          classifiers=['Programming Language :: Python',
                       'Programming Language :: Python :: 3',
                       'Operating System :: OS Independent'],

          scripts=[file for file in glob.iglob('bin/*.py')],  # Run with python -m Scripts.module args

          ext_modules=extensions,  # C extensions
          packages=packages,
          include_package_data=True,
          package_data={pkg: ['*', '*/*', '*/*/*', '*/*/*/*', '*/*/*/*/*']
                        for pkg in packages if '/' not in pkg and '\\' not in pkg},

          # Data files outside of packages
          # data_files=[('my_data', ['data/my_data.dat'])],

          # options to install extra requirements
          install_requires=[
              'pysdl2>=0.9.6',
              ],
          extras_require={
              'pygame': ['pygame>=1.9.2'],
              'qt': ['qt_thread_updater>=0.0.1', 'QtPy>=1.9.0']
              },

          # entry_points={
          #     'console_scripts': [
          #         'plot_csv=bin.plot_csv:plot_csv',
          #         ],
          #     'gui_scripts': [
          #         'baz = my_package_gui:start_func',
          #         ]
          #     }
          )
